unit MIRA_SHExt;

interface

uses
  Winapi.Windows, System.SysUtils, System.Classes, System.Win.Comserv, System.Win.ComObj,
  Winapi.ShlObj, Winapi.ActiveX;

type
  TShellExtension = class(TComObject, IShellExtInit, IContextMenu)
  private
    FPath,
    FFiles: string;
    procedure ExecuteServer(const Topic: string);
  public
    //IShellExtInit
    function Initialize(pidlFolder: PItemIDList; lpdObj: IDataObject;
      hkeyProgID: HKEY): HResult; reintroduce; stdcall;
    //IContextMenu
    function QueryContextMenu(Menu: HMENU;
      indexMenu, idCmdFirst, idCmdLast, uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd: UINT_PTR; uFlags: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; stdcall;
  end;

  TShellExtensionFactory = class (TComObjectFactory)
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

implementation

uses
  MIRA_Consts,
  Winapi.ShellAPI, AppData, RegData, Vcl.DdeMan, System.AnsiStrings, SysInfo;

{ Routine }

function ContainedPath(FileName, Path, Attrib: string): Boolean;
begin
  FileName := UpperCase(FileName);
  Path := UpperCase(Path);
  if Pos('D', Attrib) > 0 then
    Result := Pos(FileName, Path) = 1
  else
    Result := (FileName = Path) or (FileName = ExtractFileDir(Path));
end;

{ TShellExtension }

procedure TShellExtension.ExecuteServer(const Topic: string);
var
  Err: string;
  Info: TShellExecuteInfo;
  DdeClient: TDdeClientConv;
  Command: TStrings;
begin
  Err := '';
  if FindWindow('TMainForm', 'Mira') = 0 then
  begin
    FillChar(Info, SizeOf(TShellExecuteInfo), 0);
    with Info do begin
      cbSize := SizeOf(Info);
      fMask := SEE_MASK_FLAG_NO_UI or SEE_MASK_NOASYNC or SEE_MASK_WAITFORINPUTIDLE;
      lpVerb := 'open';
      lpFile := PChar(ServerFileName);
      nShow := SW_SHOWNORMAL;
    end;
    if not ShellExecuteEx(@Info) then
      Err := 'ShellExecuteEx';
  end;

  if Err = '' then
  begin
    DdeClient := TDdeClientConv.Create(nil);
    try
      if DdeClient.SetLink('Mira', Topic) then
      begin
        Command := TStringList.Create;
        try
          Command.Text := FFiles;
          if not DdeClient.ExecuteMacroLines(Command, False) then
            Err := 'ExecuteMacroLines';
        finally
          Command.Free;
        end;
      end
      else
        Err := 'SetLink';
    finally
      DdeClient.Free;
    end;
  end;

  if Err <> '' then
    MessageBox(0, PChar('FAIL: ExecuteServer.' + Err), 'Mira', MB_ICONERROR + MB_OK);
end;

function TShellExtension.Initialize(pidlFolder: PItemIDList; lpdObj: IDataObject;
  hkeyProgID: HKEY): HResult;
var
  FormatEtc: TFormatEtc;
  StgMedium: TStgMedium;
  Buffer: TStrings;
  Count, i: Integer;
  FileName: array[0..MAX_PATH] of Char;
begin
  Result := E_FAIL;
  if lpdObj = nil then Exit;

  with FormatEtc do
  begin
    cfFormat := CF_HDROP;
    ptd := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex := -1;
    tymed := TYMED_HGLOBAL;
  end;
  Result := lpdObj.GetData(FormatEtc, StgMedium);
  if Succeeded(Result) then
  begin
    Buffer := TStringList.Create;
    try
      Count := DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0);
      for i := 0 to Count - 1 do
      begin
        DragQueryFile(StgMedium.hGlobal, i, FileName, SizeOf(FileName));
        Buffer.Add(FileName);
      end;
      if Buffer.Count > 0 then
        FPath := Buffer.Strings[0]
      else
        FPath := '';
      FFiles := Buffer.Text;
    finally
      Buffer.Free;
      ReleaseStgMedium(StgMedium);
    end;
  end;
end;

function TShellExtension.QueryContextMenu(Menu: HMENU;
  indexMenu, idCmdFirst, idCmdLast, uFlags: UINT): HResult;
var
  Info: TMenuItemInfo;
  IniFile: TAppIniFile;
  Sections: TStrings;
  i: Integer;
  Seting, Source, Dest, Attribute: string;
begin
  Result := 0;
  if (uFlags and CMF_NORMAL) <> CMF_NORMAL then Exit;

  //! InsertMenu(Menu, indexMenu, MF_SEPARATOR or MF_BYPOSITION, 0, nil);
  with Info do
  begin
    cbSize :=Sizeof(TMenuItemInfo);
    fMask := MIIM_ID or MIIM_TYPE or MIIM_SUBMENU;
    ftype := MFT_STRING;
    wID := idCmdFirst;
    hSubMenu := CreateMenu;
    dwtypeData := PChar(SMNUCaption);
    cch := Length(SMNUCaption);
  end;
  if InsertMenuItem(Menu, indexMenu + 1, True, Info) then
  begin
    AppendMenu(Info.hSubMenu, MF_STRING or MF_GRAYED, idCmdFirst + 1, PChar(SBKCaption));
    AppendMenu(Info.hSubMenu, MF_STRING or MF_GRAYED, idCmdFirst + 2, PChar(SRECaption));
    Inc(Result, 3);

    IniFile := TAppIniFile.Create(AppDataName('ShaggyDog\Mira\Watches.ini'));
    try
      Sections := TStringList.Create;
      try
        IniFile.ReadSections(Sections);
        for i := 0 to Sections.Count - 1 do
        begin
          Seting := Sections.Strings[i];
          Source := IniFile.ReadString(Seting, 'Source', '');
          Dest := IniFile.ReadString(Seting, 'Dest', '');
          Attribute := IniFile.ReadString(Seting, 'Attribute', '');
          if ContainedPath(Source, FPath, Attribute) then
          begin
            EnableMenuItem(Menu, idCmdFirst + 1, MF_BYCOMMAND or MF_ENABLED);
            EnableMenuItem(Menu, idCmdFirst + 2, MF_BYCOMMAND or MF_ENABLED);
            Break;
          end
          else if ContainedPath(Dest, FPath, 'D') then
          begin
            EnableMenuItem(Menu, idCmdFirst + 2, MF_BYCOMMAND or MF_ENABLED);
            Break;
          end;
        end;
      finally
        Sections.Free;
      end;
    finally
      IniFile.Free;
    end;
  end;
end;

function TShellExtension.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
begin
  if HiWord(Integer(lpici.lpVerb)) <> 0 then
  begin
    Result := E_FAIL;
    Exit;
  end
  else
    Result := NOERROR;

  case LoWord(Integer(lpici.lpVerb)) of
    1: ExecuteServer('DdeBackupConv');
    2: ExecuteServer('DdeRestorConv');
  else
    Result := E_INVALIDARG;
    MessageBox(0, 'FAIL: IContextMenu.InvokeCommand', 'Mira', MB_ICONERROR + MB_OK);
  end;
end;

function TShellExtension.GetCommandString(idCmd: UINT_PTR; uFlags: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HResult;
begin
  Result := NOERROR;
  case idCmd of
    1: //obNAbv
      case uFlags of
        GCS_VERBA: System.AnsiStrings.StrLCopy(pszName, PAnsiChar(AnsiString(SBKHint)), cchMax - 1);
        GCS_VERBW: System.SysUtils.StrLCopy(PWideChar(pszName), PWideChar(SBKHint), cchMax - 1);
      end;
    2:  //XgA
      case uFlags of
        GCS_VERBA: System.AnsiStrings.StrLCopy(pszName, PAnsiChar(AnsiString(SREHint)), cchMax - 1);
        GCS_VERBW: System.SysUtils.StrLCopy(PWideChar(pszName), PWideChar(SREHint), cchMax - 1);
      end;
  else
    Result := E_INVALIDARG;
  end;
end;

{ TShellExtensionFactory }

procedure TShellExtensionFactory.UpdateRegistry(Register: Boolean);
var
  Reg: TRegData;
begin
  Reg := TRegData.Create;
  try
    if Register then
    begin
      if Reg.OpenKey('Software\Classes\*\shellex\ContextMenuHandlers\' + KeyName, True) then
      begin
        Reg.WriteString('', STR_ShellExtension);
        Reg.CloseKey;
      end;

      if Reg.OpenKey('Software\Classes\Directory\shellex\ContextMenuHandlers\' + KeyName, True) then
      begin
        Reg.WriteString('', STR_ShellExtension);
        Reg.CloseKey;
      end;

      if Reg.OpenKey('Software\Classes\CLSID\' + STR_ShellExtension, True) then
      begin
        Reg.WriteString('', 'Mira');
        if Reg.OpenKey('InprocServer32', True) then
        begin
          Reg.ClearValues;
          Reg.WriteString('', ModuleFileName);
          Reg.WriteString('ThreadingModel', 'Apartment');
        end;
        Reg.CloseKey;
      end;
    end
    else
    begin
      if Reg.OpenKey('Software\Classes', True) then
      begin
        Reg.DeletePath('*\shellex\ContextMenuHandlers\' + KeyName);
        Reg.DeletePath('Directory\shellex\ContextMenuHandlers\' + KeyName);
        Reg.CloseKey;
      end;

      Reg.DeleteKey('Software\Classes\CLSID\' + STR_ShellExtension);
    end;
  finally
    Reg.Free;
  end;
end;

initialization
  TShellExtensionFactory.Create(ComServer, TShellExtension, GUID_ShellExtension,
    'Mira', 'MiraSHExt', ciMultiInstance, tmApartment);

end.
