unit MIRA_Filer;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.StdCtrls, Vcl.ComCtrls, AfViews;

type
  TFilerForm = class(TForm)
    ToolPanel: TPanel;
    OKButton: TButton;
    CancelButton: TButton;
    OverwriteChk: TCheckBox;
    FileView: TTreeViewAf;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FileViewExpanding(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
    procedure FileViewCheckStateChanged(Sender: TCustomTreeView; Node: TTreeNode;
      CheckState: TNodeCheckState);
    procedure FileViewDeletion(Sender: TObject; Node: TTreeNode);
  private
    { Private 錾 }
    procedure FillNode(ANode: TTreeNode; APath: string; AFunc: Integer; AIndex: Integer);
    procedure CheckChildren(Node: TTreeNode; State: Boolean);
  public
    { Public 錾 }
    procedure Init(Files: TStrings);
  end;

var
  FilerForm: TFilerForm;

implementation

{$R *.dfm}

uses
  MIRA_Main,
  Winapi.ShlObj, ShellTypes, ShellUtils, Winapi.ShellAPI, Winapi.CommCtrl, System.IOUtils;

{ Routine }

function SortProc(Node1, Node2: TTreeNode; Param: Integer): Integer; stdcall;
var
  ID1, ID2: PItemIDList;
begin
  ID1 := ParseDisplayName(DesktopShellFolder, PSyncOpStruct(Node1.Data)^.Path);
  ID2 := ParseDisplayName(DesktopShellFolder, PSyncOpStruct(Node2.Data)^.Path);
  try
    Result := DesktopShellFolder.CompareIDs(0, ID1, ID2);
  finally
    DisposePIDL(ID1);
    DisposePIDL(ID2);
  end;
end;

{ TFilerForm }

procedure TFilerForm.FillNode(ANode: TTreeNode; APath: string; AFunc: Integer; AIndex: Integer);
var
  SyncOp: PSyncOpStruct;
  ID: PItemIDList;
begin
  New(SyncOp);
  with SyncOp^ do
  begin
    Path := APath;
    Func := AFunc;
    Index := AIndex;
  end;
  ID := ParseDisplayName(DesktopShellFolder, APath);
  try
    with ANode do
    begin
      Data := Pointer(SyncOp);
      ImageIndex := GetShellImage(ID, False, False);
      SelectedIndex := GetShellImage(ID, False, True);
      if SelectedIndex = 0 then
        SelectedIndex := ImageIndex;
      HasChildren := TDirectory.Exists(APath) and not TDirectory.IsEmpty(APath);
    end;
  finally
    DisposePIDL(ID);
  end;
end;

procedure TFilerForm.CheckChildren(Node: TTreeNode; State: Boolean);
var
  i: Integer;
begin
  for i := 0 to Node.Count - 1 do
    Node.Item[i].Checked := State;
end;

procedure TFilerForm.FormCreate(Sender: TObject);
const
  InfoFlags = SHGFI_SYSICONINDEX or SHGFI_SMALLICON;
var
  ImageListHandle: THandle;
  FileInfo: TSHFileInfo;
begin
  ImageListHandle := SHGetFileInfo('.', 0, FileInfo, SizeOf(FileInfo), InfoFlags);
  SendMessage(FileView.Handle, TVM_SETIMAGELIST, TVSIL_NORMAL, ImageListHandle);
end;

procedure TFilerForm.Init(Files: TStrings);
var
  Item: TListItem;
  Watcher: TWatcher;
  FileName, Path, Pattern, Entrity: string;
  NewNode: TTreeNode;
begin
  FileView.Items.Clear;
  for Item in MainForm.WatchView.Items do
  begin
    Watcher := TWatcher(Item.Data);
    for FileName in Files do
      if ContainedPath(Watcher.Source, FileName, Watcher.Attrib) then
      begin
        if FileExists(FileName) then
        begin
          Path := ExtractFileDir(StringReplace(FileName, Watcher.Source, Watcher.Dest, []));
          Pattern := Format('%s.*%s', [ExtractFileName(FileName), ExtractFileExt(FileName)]);
          for Entrity in TDirectory.GetFileSystemEntries(Path, Pattern) do
          begin
            NewNode := FileView.Items.Add(nil, ExtractFileName(Entrity));
            FillNode(NewNode, Entrity, FO_COPY, Item.Index);
          end;
        end
        else
        begin
          NewNode := FileView.Items.Add(nil, ExtractFileName(FileName));
          FillNode(NewNode,
            StringReplace(FileName, Watcher.Source, Watcher.Dest, []), FO_COPY, Item.Index);
        end;
      end
      else if ContainedPath(Watcher.Dest, FileName, faDirectory) then
      begin
        NewNode := FileView.Items.Add(nil, ExtractFileName(FileName));
        FillNode(NewNode, FileName, FO_COPY, Item.Index);
      end;
  end;
  FileView.CustomSort(@SortProc, 0);
end;

procedure TFilerForm.FormShow(Sender: TObject);
begin
  OKButton.Enabled := FileView.ChkCount <> 0;
end;

procedure TFilerForm.FileViewExpanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
var
  SyncOp: TSyncOpStruct;
  Entrity: string;
  NewNode: TTreeNode;
begin
  if Node.HasChildren and (Node.Count = 0) then
  begin
    SyncOp := PSyncOpStruct(Node.Data)^;
    for Entrity in TDirectory.GetFileSystemEntries(SyncOp.Path, '*') do
    begin
      NewNode := FileView.Items.AddChild(Node, ExtractFileName(Entrity));
      FillNode(NewNode, Entrity, SyncOp.Func, SyncOp.Index);
    end;
    Node.CustomSort(@SortProc, 0);

    if Node.Checked then
      CheckChildren(Node, True);
  end;
end;

procedure TFilerForm.FileViewCheckStateChanged(Sender: TCustomTreeView; Node: TTreeNode;
  CheckState: TNodeCheckState);
var
  ParentNode: TTreeNode;
begin
  CheckChildren(Node, Node.Checked);
  if CheckState = ncsUnchecked then
  begin
    FileView.OnCheckStateChanged := nil;
    try
      ParentNode := Node.Parent;
      while Assigned(ParentNode) do
      begin
        ParentNode.Checked := False;
        ParentNode := ParentNode.Parent;
      end;
    finally
      FileView.OnCheckStateChanged := FileViewCheckStateChanged;
    end;
  end;

  OKButton.Enabled := FileView.ChkCount > 0;
end;

procedure TFilerForm.FileViewDeletion(Sender: TObject; Node: TTreeNode);
begin
  Dispose(PSyncOpStruct(Node.Data));
end;

end.
