unit MIRA_Seting;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, RefCtrls, NumEdits;

type
  TSetingDialog = class(TForm)
    OKButton: TButton;
    CancelButton: TButton;
    TitleLbl: TLabel;
    TitleEdt: TEdit;
    SourceLbl: TLabel;
    SourceEdt: TRefDirectoryEdit;
    DestLbl: TLabel;
    DestEdt: TRefDirectoryEdit;
    IncludeLbl: TLabel;
    IncludeEdt: TEdit;
    ExcludeLbl: TLabel;
    ExcludeEdt: TEdit;
    AttributeGrp: TGroupBox;
    ReadOnlyChk: TCheckBox;
    HiddenChk: TCheckBox;
    SysFileChk: TCheckBox;
    DirectoryChk: TCheckBox;
    ArchiveChk: TCheckBox;
    SymLinkChk: TCheckBox;
    KeepingLbl: TLabel;
    KeepingEdt: TUpDownEdit;
    StoringLbl: TLabel;
    StoringEdt: TUpDownEdit;
    OptionGrp: TGroupBox;
    UpdateDirTooChk: TCheckBox;
    ToBitbucketChk: TCheckBox;
    WarningLbl: TLabel;
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure EdtChange(Sender: TObject);
    procedure EdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
      Relation: TNumericallyRelationship);
  private
    { Private 錾 }
    procedure UpdateOKButton;
  public
    { Public 錾 }
  end;

var
  SetingDialog: TSetingDialog;

implementation

{$R *.dfm}

uses
  MIRA_Main,
  Vcl.ComCtrls, FileUtils;

resourcestring
  STitle = '^Cg';
  SSource = '';
  SDest = '';
  SInclude = 'Ώۃ}XN';
  SKeeping = 'ۊǓ';
  SStoring = 'ۊǌ';
  SUnnamed = '%sݒł';
  SSame = '%s%sł';
  SDuplicate = '%sق̍ڂƏdĂ܂';
  SContains = '%s%sɊ܂܂Ă܂';
  SAccord = 's%sł';

procedure TSetingDialog.UpdateOKButton;
var
  ErrorMsg: string;
  StartIndex: Integer;
  Item: TListItem;
begin
  ErrorMsg := '';
  //CheckTitle
  if TitleEdt.Text = '' then
    ErrorMsg := Format(SUnnamed, [STitle])
  else
  begin
    StartIndex := MainForm.WatchView.ItemFocused.Index;
    Item := MainForm.WatchView.FindCaption(StartIndex, TitleEdt.Text, False, False, True);
    if Assigned(Item) and (Item.Index <> StartIndex) then
      ErrorMsg := Format(SDuplicate, [STitle]);
  end;
  //CheckPath
  if ErrorMsg = '' then
  begin
    if SourceEdt.Text = '' then
      ErrorMsg := Format(SUnnamed, [SSource])
    else if DestEdt.Text = '' then
      ErrorMsg := Format(SUnnamed, [SDest])
    else if SameText(SourceEdt.Text, DestEdt.Text) then
      ErrorMsg := Format(SSame, [SSource, SDest])
    else if DirectoryChk.Checked and ContainedPath(SourceEdt.Text, DestEdt.Text, faDirectory) then
      ErrorMsg := Format(SContains, [SDest, SSource])
    else
    begin
      for Item in MainForm.WatchView.Items do
        if (DestEdt.Text = TWatcher(Item.Data).Dest) and not Item.Focused then
        begin
          ErrorMsg := Format(SDuplicate, [SDest]);
          Break;
        end;
    end;
  end;
  //CheckMask
  if ErrorMsg = '' then
  begin
    if IncludeEdt.Text = '' then
      ErrorMsg := Format(SUnnamed, [SInclude]);
  end;
  //CheckKeeping
  if ErrorMsg = '' then
  begin
    if KeepingEdt.Relation <> nrAccord then
      ErrorMsg := Format(SAccord, [SKeeping]);
    if StoringEdt.Relation <> nrAccord then
      ErrorMsg := Format(SAccord, [SStoring]);
  end;

  WarningLbl.Caption := ErrorMsg;
  OKButton.Enabled := WarningLbl.Caption = ''
end;

procedure TSetingDialog.FormShow(Sender: TObject);
begin
  with TWatcher(MainForm.WatchView.ItemFocused.Data) do
  begin
    TitleEdt.Text := Title;
    SourceEdt.Text := Source;
    DestEdt.Text := Dest;
    IncludeEdt.Text := Includes;
    ExcludeEdt.Text := Excludes;

    ReadOnlyChk.Checked := AttrInSet(faReadOnly, Attrib);
    HiddenChk.Checked := AttrInSet(faHidden, Attrib);
    SysFileChk.Checked := AttrInSet(faSysFile, Attrib);
    DirectoryChk.Checked := AttrInSet(faDirectory, Attrib);
    ArchiveChk.Checked := AttrInSet(faArchive, Attrib);
    SymLinkChk.Checked := AttrInSet(faSymLink, Attrib);

    KeepingEdt.Value := Keeping;
    StoringEdt.Value := Storing;

    UpdateDirTooChk.Checked := woUpdateDirToo in Options;
    ToBitbucketChk.Checked := woToBitbucket in Options;
  end;
  UpdateOKButton;
end;

procedure TSetingDialog.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  Attr: Integer;
  Opt: TWatchOptions;
begin
  if ModalResult = mrOK then
    with TWatcher(MainForm.WatchView.ItemFocused.Data) do
    begin
      Title := TitleEdt.Text;
      Source := SourceEdt.Text;
      Dest := DestEdt.Text;
      Includes := IncludeEdt.Text;
      Excludes := ExcludeEdt.Text;

      Attr := 0;
      if ReadOnlyChk.Checked  then Inc(Attr, faReadOnly);
      if HiddenChk.Checked then Inc(Attr, faHidden);
      if SysFileChk.Checked then Inc(Attr, faSysFile);
      if DirectoryChk.Checked then Inc(Attr, faDirectory);
      if ArchiveChk.Checked then Inc(Attr, faArchive);
      if SymLinkChk.Checked then Inc(Attr, faSymLink);
      Attrib := Attr;

      Keeping := KeepingEdt.Value;
      Storing := StoringEdt.Value;

      Opt := [];
      if UpdateDirTooChk.Checked then Include(Opt, woUpdateDirToo);
      if ToBitbucketChk.Checked then Include(Opt, woToBitbucket);
      Options := Opt;
    end;
end;

procedure TSetingDialog.EdtChange(Sender: TObject);
begin
  UpdateOKButton;
end;

procedure TSetingDialog.EdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
  Relation: TNumericallyRelationship);
begin
  UpdateOKButton;
end;

end.
